import json

from db import models, ForeignKey, createsession, engine, attach_session
from auth import User
from core import defaults

from datetime import datetime

from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound
from sqlalchemy import Column, String, Numeric, DateTime, Text, Boolean

class Account(models.Model):
	name = Column(String(50))
	description = Column(Text)
	created = Column(DateTime, default=datetime.utcnow)
	active = Column(Boolean, default=True)
	public = Column(Boolean, default=False)
	acc_class = ForeignKey(AccountClass, default=None, nullable=True)
	currency = ForeignKey(Currency)

	@attach_session
	def transfer_funds(self, user, dest, amount, description='', session):
		if amount <= 0.0:
			raise Exception('Amount must be greater than zero.')

		# check if user is owner
		try:
			account_owner = session.query(AccountOwner).filter_by(account=self, user=user, active=True).one()
		except NoResultFound:
			raise Exception('User not allowed.')
		except MultipleResultsFound:
			raise Exception('Auth conflict.')

		# check balance
		try:
			balance = session.query(AccountBalance.balance).filter_by(account=self).order_by('timestamp').first()
		except NoResultFound:
			raise Exception('Balance not found.')

		if balance < amount:
			raise Exception('Not enough balance.')

		# make the transfer
		movement = Movement(orig=self, dest=dest, user=user, amount=amount, description=description)
		session.add(movement)

		movement.update_balance(session)

		return movement

	@attach_session
	def getbalance(self, session):

		try:
			balance = session.query(AccountBalance.balance).filter_by(account=self).order_by('timestamp').first()
		except NoResultFound:
			raise Exception('Balance not found.')
		return balance

	@attach_session
	def is_owner(self, user, session):

		try:
			account_owner = session.query(AccountOwner).filter_by(user=user, account=self).one()
		except:
			return False
		return True

	@attach_session
	def history(self, last=15, since=None, session):
		
		try:
			if since:
				accounthistory = session.query(AccountBalance).filter_by(account=self, timestamp > since).order_by('-timestamp')
			else:
				accounthistory = session.query(AccountBalance).filter_by(account=self).order_by('-timestamp').limit(last)
			return accounthistory
		except Exception, e:
			raise e


class AccountOwner(models.Model):
	user = ForeignKey(User)
	account = ForeignKey(Account)
	active = Column(Boolean, default=True)
	created = Column(DateTime, default=datetime.utcnow)
	deactivated = Column(DateTime, default=None, nullable=True)


class AccountBalance(models.Model):
	account = ForeignKey(Account)
	movement = ForeignKey(Movement)
	balance = Column(Numeric(defaults.DECIMAL_PRECISION, defaults.DECIMAL_SCALE))
	timestamp = Column(DateTime, default=datetime.utcnow)


class AccountClass(models.Model):
	name = Column(String(50))
	info = Column(Text, default='')

	def get(self, key):
		try:
			data = json.loads(info)
		except:
			data = {}

		if key not in data:
			return None

		return data[key]

	def set(self, key, value):
		try:
			data = json.loads(info)
		except:
			data = {}

		data[key] = value

		self.info = json.dumps(data)


class Movement(models.Model):
	orig = ForeignKey(Account, nullable=True)
	dest = ForeignKey(Account, nullable=True)
	user = ForeignKey(User, nullable=True)
	amount = Column(Numeric(defaults.DECIMAL_PRECISION, defaults.DECIMAL_SCALE))
	description = Column(Text)
	timestamp = Column(DateTime, default=datetime.utcnow)

	@attach_session
	def update_balance(self, session):

		if self.orig is not None:
			orig_balance = session.query(AccountBalance).filter_by(account=self.orig).order_by(AccountBalance.timestamp.desc()).first().balance
			orig_balance_new = AccountBalance(account=self.orig, movement=self, balance=(orig_balance - self.amount))
			session.add(orig_balance_new)

		if self.dest is not None:
			try:
				dest_balance = session.query(AccountBalance).filter_by(account=self.dest).order_by(AccountBalance.timestamp.desc()).first.balance
			except Exception, e:
				dest_balance = 0.0

			dest_balance_new = AccountBalance(account=self.dest, movement=self, balance=(dest_balance + self.amount))
			session.add(dest_balance_new)


class Currency(models.Model):
	name = Column(String(50))
	singular = Column(String(50))
	plural = Column(String(50))
	stock = Column(Numeric(defaults.DECIMAL_PRECISION, defaults.DECIMAL_SCALE))