from db.models import Model
from db import createsession, engine
from auth import auth_required, User, Tag, has_tag

from accounts import Account, Currency, AccountClass, AccountBalance, Movement, AccountOwner

from core import error, success, defaults
from utils import required_fields, jsonresponse

from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound

from cdecimal import Decimal

@auth_required
@has_tag('admin')
@required_fields('account', 'amount', 'description')
def create_value(request, data, user):
	account_name = data['account']
	amount = Decimal(data['amount'])
	description = data['description']

	if amount < 0:
		return error(request, 'Amount must be greater than zero.')

	with createsession(engine) as session:
		try:
			account = session.query(Account).filter_by(name=account_name).one()
		except NoResultFound:
			return error(request, 'Invalid account.')
		except MultipleResultsFound:
			return error(request, 'Account error.')

		movement = Movement(orig=None, dest=account, amount=amount, description=description, user=user)
		session.add(movement)

		movement.update_balance(session)

		# update total stock
		currency = account.currency
		currency.stock = currency.stock + amount
		session.add(currency)

		session.commit()

	return success()

@auth_required
@has_tag('admin')
@required_fields('username', 'tag')
def add_user_tag(request, data, user):
	username = data['user']
	tag = data['tag']

	with createsession(engine) as session:
		try:
			target_user = session.query(User).filter_by(username=username).one()
		except:
			return error(request, 'User error.')

		try:
			target_user.set_tag(tag, session)
		except Exception, e:
			return error(request, e.message)

	return success()



@auth_required
@has_tag('admin')
@required_fields('username', 'tag')
def remove_user_tag(request, data, user):
	username = data['user']
	tag = data['tag']

	with createsession(engine) as session:
		try:
			target_user = session.query(User).filter_by(username=username).one()
		except:
			return error(request, 'User error.')

		try:
			target_user.delete_tag(tag, session)
		except Exception, e:
			return error(request, e.message)

	return success()


@auth_required
@has_tag('admin')
@required_fields('name', 'info')
def create_tag(request, data, user):
	name = data['name']
	info = data['info']

	with createsession(engine) as session:
		try:
			tag = session.query(Tag).filter_by(name=name).one()
			return error('request', "Tag '%s' already exists." % name)
		except NoResultFound:
			tag = Tag(name, info)
		
		session.add(tag)
		session.commit()
	
	return success()

@auth_required
@has_tag('admin')
@required_fields('name', 'info')
def update_tag(request, data, user):
	name = data['name']
	info = data['info']

	with createsession(engine) as session:
		try:
			tag = session.query(Tag).filter_by(name=name).one()
		except:
			return error('request', 'Tag not found.')
		
		tag.info = info
		session.add(tag)
		session.commit()
	
	return success()

@auth_required
@has_tag('admin')
def list_tag(request, data, user):
	
	tag_list = []
	
	with createsession(engine) as session:
		try:
			tags = session.query(Tag)
		except Exception, e:
			raise error(request, e.message)

		for tag in tags:
			tag_list.append({
				'tag': tag.name,
				'info': tag.info
				})

	return jsonresponse(tag_list)


@auth_required
@has_tag('admin')
@required_fields('name', 'singular', 'plural')
def create_currency(request, data, user):
	name = data['name']
	singular = data['singular']
	plural = data['plural']
	
	with createsession(engine) as session:
		currency = Currency(name=name, singular=singular, plural=plural, stock=Decimal('0.0'))
		session.add(currency)
		session.commit()

	return success()

@auth_required
@has_tag('admin')
@required_fields('name', 'singular', 'plural')
def update_currency(request, data, user):
	name = data['name']
	singular = data['singular']
	plural = data['plural']
	
	with createsession(engine) as session:
		try:
			currency = session.query(Currency).filter_by(name=name).one
		except:
			return error(request, 'Currency error.')
		
		currency.name = name
		currency.singular = singular
		currency.plural = plural

		session.add(currency)
		session.commit()

	return success()

@auth_required
@has_tag('admin')
def list_currency(request, data, user):
	currency_list = []

	with createsession(engine) as session:
		try:
			currencies = session.query(Currency)
		except:
			return error(request, 'Currency error.')

		for currency in currencies:
			currency_list.append({
				'currency': currency.name,
				'singular': currency.singular,
				'plural': currency.plural,
				'stock': currency.stock
				})

	return jsonresponse(currency_list)