from db.models import Model
from db import createsession, engine
from auth import auth_required, User, Tag, has_tag

from accounts import Account, Currency, AccountClass, AccountBalance, Movement, AccountOwner

from core import error, success, defaults
from utils import required_fields, jsonresponse

from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound

from decimal import Decimal

from models import CurrencyWallet, AccountWallet, IncomingTransactions, MovementQueue
from blockchain import ACCOUNT_PREFIX, account_class
from protocol import resolve_federation, post_data
from blockchainapi import get_new_wallet

@auth_required
@has_tag('admin')
@required_fields('currency', 'identifier', 'password', 'defaultrate')
def set_currency_wallet(request, data, user):
	currency_name = data['currency']
	identifier = data['identifier']
	password = data['password']
	defaultrate = Decimal(data['defaultrate'])

	with createsession(engine) as session:
		try:
			currency = session.query(Currency).filter_by(name=currency_name).one()
		except:
			return error(request, 'Currency error')

		try:
			currencywallet = session.query(CurrencyWallet).filter_by(currency=currency).one()
		except MultipleResultsFound:
			for currencywallet_item in currencywallet:
				session.delete(currencywallet_item)

			currencywallet = CurrencyWallet(currency=currency, identifier=identifier, password=password, defaultrate=defaultrate)
		except NoResultFound:
			currencywallet = CurrencyWallet(currency=currency, identifier=identifier, password=password, defaultrate=defaultrate)

		session.add(currencywallet)
		session.commit()

	return success()

@auth_required
@has_tag('admin')
@required_fields('currency')
def get_currency_wallet(request, data, user):
	currency_name = data['currency']

	with createsession(engine) as session:
		try:
			currency = session.query(Currency).filter_by(name=currency_name).one()
		except:
			return error(request, 'Currency error')

		try:
			currencywallet = session.query(CurrencyWallet).filter_by(currency=currency).one()
		except:
			return error(request, 'Currency wallet error.')

		return jsonresponse({
				"identifier": currencywallet.identifier,
				"password": currencywallet.password,
				"defaultrate": currencywallet.defaultrate,
			})

@auth_required
@required_fields('currency')
def get_currency_rate(request, data, user):
	currency_name = data['currency']

	response = {}

	with createsession(engine) as session:
		try:
			currency = session.query(Currency).filter_by(name=currency_name).one()
		except:
			return error(request, 'Currency error')

		try:
			currencywallet = session.query(CurrencyWallet).filter_by(currency=currency).one()
		except:
			return error(request, 'Currency wallet error.')

		response['currency'] = currency_name
		response['rate'] = currencywallet.get_rate(session)

	return jsonresponse(response)


@auth_required
@required_fields('orig', 'dest', 'amount')
def send(request, data, user):
	orig = data['orig']
	dest = data['dest']
	amount = Decimal(data['amount'])
	description = data['description'] if 'description' in data else ''

	if amount < 0.0:
		return error(request, 'Amount must be greater than zero.')

	with createsession(engine) as session:
		try:
			acc_orig = session.query(Account).filter_by(name=orig).one()
		except NoResultFound:
			return error(request, 'Invalid origin account.')
		except MultipleResultsFound:
			return error(request, 'Account error.', http_code=500)

		# check ownership
		if not acc_orig.is_owner(user, session):
			return error(request, 'Not authorized.')

		# check amount
		if acc_orig.getbalance() < amount:
			return error(request, 'Not enough balance.')

		# get transit account
		transit_acc_name = ACCOUNT_PREFIX + orig.currency.name + '_' + orig
		try:
			transit_acc = session.query(Account).filter_by(name=transit_acc_name).one()
		except NoResultFound:
			transit_acc = Account(name=transit_acc_name, acc_class=account_class, currency=acc_orig.currency)
			session.add(transit_acc)
		except MultipleResultsFound:
			return error(request, 'Account error.')

		# check federation
		try:
			destinfo = resolve_federation(dest)
		except Exception, e:
			return error(request, e.message)

		if isinstance(destinfo, str):
			return error(request, 'Local account.')

		# send to transit account
		movement = acc_orig.transfer_funds(user, transit_acc, amount, description, session)

		# put on the queue
		MovementQueue.put('new', movement=movement, orig=orig, dest=dest, user=user, session=session)

		session.commit()

	response = { "success": 1, "status": "on queue" }

	return jsonresponse(response)

@required_fields('account')
def get_receive_address(request, data):
	account_name = data['account']

	response = {}

	with createsession(engine) as session:
		try:
			account = session.query(Account).filter_by(name=account_name).one()
		except NoResultFound:
			return error(request, 'Invalid account.')
		except MultipleResultsFound:
			return error(request, 'Account error.')

		generate_new = False

		try:
			accountwallet = session.query(AccountWallet).filter_by(account=account_name).first()
		except NoResultFound:
			generate_new = True

		if generate_new:
			currency = account.currency
			address = get_new_wallet(currency, account_name)

			accountwallet = AccountWallet(account_name, address)
			session.add(accountwallet)
			session.commit()

		response['account'] = accountwallet.account
		response['address'] = accountwallet.address


