from db.models import Model
from db import createsession, engine
from auth import auth_required, User, Tag

from accounts import Account, Currency, AccountClass, AccountBalance, Movement, AccountOwner

from core import error, success, defaults
from utils import required_fields, jsonresponse

from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound

from decimal import Decimal


@required_fields('username', 'password')
def login(request, data):
	
	with createsession(engine) as session:

		username = data['username']
		password = data['password']

		try:
			user = session.query(User).filter_by(username=username).one()
		except NoResultFound:
			return error(request, 'Incorrect username and/or password.', http_code=403)
		except MultipleResultsFound:
			return error(request, 'Auth error.', http_code=500)

		if not user.check_password(password):
			return error(request, 'Incorrect username and/or password.', http_code=403)

		key = user.create_session_key()

	return jsonresponse({'key': key})

@auth_required
@required_fields('account', 'currency')
def create_account(request, data, user):
	account_name = data['account']
	currency_name = data['currency']
	description = data['description'] if 'description' in data else ''
	accountclass = data['accountclass'] if 'accountclass' in data else ''
	public = data['public'] if 'public' in data else 0

	with createsession(engine) as session:

		# check currency
		try:
			currency = session.query(Currency).filter_by(name=currency_name).one()
		except NoResultFound:
			return error(request, 'Incorrect currency name.')
		except MultipleResultsFound:
			return error(request, 'Currency error.', http_code=500)

		# check account class
		try:
			if len(accountclass) > 0:
				acc_class = session.query(AccountClass).filter_by(name=accountclass).one()
			else:
				acc_class = None
		except NoResultFound:
			return error(request, 'Incorrect account class.')
		except MultipleResultsFound:
			return error(request, 'Account class error.', http_code=500)

		# check account name
		try:
			account = session.query(Account).filter_by(name=account_name).one()

			return error(request, "Account '%s' already exists." % account_name)
		except MultipleResultsFound:
			return error(request, 'Account error.', http_code=500)
		except NoResultFound:
			account = Account(name=account_name, description=description, currency=currency, acc_class=acc_class)
			session.add(account)

		# create account ownership
		accountowner = AccountOwner(user=user, account=account)
		session.add(accountowner)
		
		# create initial balance
		movement = Movement(orig=None, dest=account, user=user, amount=Decimal('0.00'), description='Account creation.')
		session.add(movement)

		movement.update_balance(session=session)

		session.commit()

	return success()

@required_fields('username', 'password', 'email')
def create_user(request, data):

	with createsession(engine) as session:
		new_user = User(username=data['username'], email=data['email'])
		new_user.change_password(data['password'])

		session.add(new_user)
		session.commit()

	return success()

@auth_required
@required_fields('orig', 'dest', 'amount')
def transfer_funds(request, data, user):
	orig_name = data['orig']
	dest_name = data['dest']
	amount = Decimal(data['amount'])
	description = data['description'] if 'description' in data else ''

	with createsession(engine) as session:
		# check if orig exists
		try:
			orig = session.query(Account).filter_by(name=orig_name).one()
		except NoResultFound:
			return error(request, 'Origin account not found.')
		except MultipleResultsFound:
			return error(request, 'Orig error.', http_code=500)

		# check if user is owner
		if not orig.is_owner(user, session=session):
			return error(request, "You are not the owner of the account '%s'." % orig_name)

		# check if dest exists
		try:
			dest = session.query(Account).filter_by(name=dest_name).one()
		except NoResultFound:
			return error(request, 'Destination account not found.')
		except MultipleResultsFound:
			return error(request, 'Dest error.', http_code=500)

		# make transfer
		try:
			orig.transfer_funds(user=user, dest=dest, amount=amount, description=description, session=session)
		except Exception, e:
			return error(request, e.message)

		session.commit()

	return success()
	

@auth_required
@required_fields('account')
def account_history(request, data, user):
	account_name = data['account']
	last = data['last'] if 'last' in data else None
	since = datetime.utcfromtimestamp(data['since']) if 'since' in data else None

	account_hist = []

	with createsession(engine) as session:

		# check if account exists
		try:
			account = session.query(Account).filter_by(name=account_name).one()
		except NoResultFound:
			return error(request, 'Invalid account.')

		# check if user is owner
		if not account.is_owner(user, session=session):
			return error(request, "You are not the owner of the account '%s'." % orig_name)

		# get history
		if since:
			history = account.history(since=since)
		elif last:
			history = account.history(last=last)
		else:
			history = account.history()

		for balance in history:
			account_hist.append({
				"timestamp": balance.timestamp,
				"balance": balance.balance,
				"orig": balance.movement.orig.name,
				"dest": balance.movement.dest.name,
				"description": balance.movement.description,
				"amount": balance.movement.amount
				})
	
	return jsonresponse(account_hist)

@auth_required
def list_accounts(request, data, user):

	account_list = []

	with createsession(engine) as session:
		accounts = session.query(AccountOwner).filter_by(user=user)
		
		for account in accounts:
			account_list.append({
				"account": account.name,
				"description": account.description,
				"balance": account.getbalance(),
				"currency": account.currency.name
			})
		
	return jsonresponse(account_list)

@auth_required
def list_currency(request, data, user):

	currency_list = []

	with createsession(engine) as session:
		currencies = session.query(Currency)

		for currency in currencies:
			currency_list.append({
				"name": currency.name,
				"singular": currency.singular,
				"plural": currency.plural,
				"stock": currency.stock
				})

	return jsonresponse(currency_list)

@auth_required
@required_fields('tag')
def has_tag(request, data, user):
	tag_name = data['tag']

	response['value'] = True

	if not user.has_tag(tag):
		response['value'] = False

	return jsonresponse(response)
